import Head from 'next/head'
import Link from 'next/link'
import styled, { createGlobalStyle } from 'styled-components'
import { Folder, Settings, Trash2 } from 'react-feather'
import Button from '../../molecules/Button'
import TopBar from '../../organisms/widgets/TopBar'
import FolderItem from '../../organisms/tree/FolderItem'

const Variables = createGlobalStyle({
	':root': {
		'--size-sidebar': '360px',
	},
})

const Container = styled('div')({
	maxWidth: 720,
	padding: '0 1rem',
	boxSizing: 'border-box',
})

const SideBar = styled('div')({
	width: 'calc(50% - var(--size-sidebar) * 0.5)',
	backgroundColor: 'var(--color-bg, white)',
	position: 'fixed',
	top: 0,
	left: 0,
	height: '100%',
})

const SideBarContainer = styled('div')({
	marginLeft: 'auto',
	width: 'var(--size-sidebar)',
})

const Main = styled('div')({
	paddingLeft: 'calc(50% - var(--size-sidebar) * 0.5)',
})

const FolderInfoContainer = styled(Container)({
	display: 'flex',
	justifyContent: 'space-between',
	height: '4rem',
	alignItems: 'center',
})

const SidebarLink = styled('a')({
	display: 'flex',
	height: '4rem',
	alignItems: 'center',
})

const InnerSidebar = styled('div')({
	width: 'calc(var(--size-sidebar) - 4rem)',
	height: '100%',
	position: 'absolute',
	top: 0,
	right: 0,
	backgroundColor: 'var(--color-bg, white)',
})

const SideBarIcon = styled('div')({
	width: '4rem',
	height: '4rem',
	display: 'grid',
	placeContent: 'center',
})

const SidebarContents = styled('div')({
	width: '100%',
	height: '100%',
	position: 'relative',
})

const Tree = styled('div')({
	width: '100%',
	height: '100%',
	overflow: 'auto',
})

const TreeParent = styled('ul')({
	margin: 0,
	padding: 0,
})

const TreeChild = styled('li')({
	display: 'block',
})

const TreeActions = styled('div')({
	position: 'sticky',
	display: 'flex',
	justifyContent: 'flex-end',
	alignItems: 'center',
	top: 0,
	left: 0,
	width: '100%',
	height: '4rem',
	backgroundColor: 'var(--color-bg, white)',
	padding: '0 1rem',
	boxSizing: 'border-box',
})

const TreeLink = styled('a')({
	display: 'inline-flex',
	verticalAlign: 'top',
	height: '2rem',
	alignItems: 'center',
	whiteSpace: 'nowrap',
	padding: '0 1rem',
	boxSizing: 'border-box',
	[`+ ${TreeParent}`]: {
		paddingLeft: '1.5rem',
	},
})

const FolderGrid = styled('div')({
	display: 'grid',
	gridTemplateColumns: 'repeat(3, 1fr)',
	gap: '0.5rem',
	margin: '2rem 0 4rem',
})

const FolderGridCell = styled('div')({
	paddingBottom: '100%',
	position: 'relative',
})

const FolderGridContents = styled('div')({
	position: 'absolute',
	top: 0,
	left: 0,
	width: '100%',
	height: '100%',
})

const Header = styled('div')({
	position: 'sticky',
	top: 'var(--size-topbar)',
	left: 0,
	width: '100%',
	zIndex: 2,
	backgroundColor: 'var(--color-bg, white)',
})

const renderChild = c => (
	<TreeChild
		key={c.id}
	>
		<TreeLink>
			{c.name}
			</TreeLink>
		<TreeParent>
			{c.children.map(renderChild)}
		</TreeParent>
	</TreeChild>
)

const FolderTemplate = ({
	path = [],
	query = '',
	id = '',
	children,
	items = [],
}) => {
	return (
		<>
			<Head>
				<title>{ Array.isArray(path) && path.length > 0 ? `${path.slice(-1)[0]} | Bruhbot` : 'Bruhbot' }</title>
				<link rel="icon" href="/favicon.ico" />
			</Head>
			<Variables />
			<TopBar
				query={query}
			/>
			<SideBar>
				<SidebarContents>
					<Link
						href={{
							pathname: '/my/folders',
						}}
						passHref
					>
						<SidebarLink>
							<SideBarContainer>
								<SideBarIcon>
									<Folder />
								</SideBarIcon>
							</SideBarContainer>
						</SidebarLink>
					</Link>
					<Link
						href={{
							pathname: '/my/bin',
						}}
						passHref
					>
						<SidebarLink>
							<SideBarContainer>
								<SideBarIcon>
									<Trash2 />
								</SideBarIcon>
							</SideBarContainer>
						</SidebarLink>
					</Link>
					<Link
						href={{
							pathname: '/my/settings',
						}}
						passHref
					>
						<SidebarLink>
							<SideBarContainer>
								<SideBarIcon>
									<Settings />
								</SideBarIcon>
							</SideBarContainer>
						</SidebarLink>
					</Link>
					<InnerSidebar>
						<Tree>
							<TreeActions>
								<Button>
									New Folder
								</Button>
							</TreeActions>
							<TreeParent>
								{children.map(renderChild)}
							</TreeParent>
						</Tree>
					</InnerSidebar>
				</SidebarContents>
			</SideBar>
			<Main>
				<Header>
					<FolderInfoContainer>
						<div>
							Folder ID: {id}
						</div>
						<div>
							<Button>
								Delete
							</Button>
						</div>
					</FolderInfoContainer>
				</Header>
				<Container>
					<FolderGrid>
						{
							items.map(i => (
								<FolderGridCell
									key={i.id}
								>
									<FolderGridContents>
										<FolderItem
											name={i.name}
											url={i.url}
										/>
									</FolderGridContents>
								</FolderGridCell>
							))
						}
					</FolderGrid>
				</Container>
			</Main>
		</>
	)
}

export default FolderTemplate
